from kodi_addon_checker.report import Record
from kodi_addon_checker.reporter import Reporter, reporter


@reporter(name="array", enabled=False)
class ArrayReporter(Reporter):
    def __init__(self):
        self.reports = []

    def report(self, report):
        if type(report) is Record:
            self.reports.append(report)



import os
import kodi_addon_checker.check_addon as check_addon
from .record import INFORMATION, PROBLEM, Record
from .report import Report


def check_repo(repo_path, branch_name, all_repo_addons, pr, config):
    repo_report = Report(repo_path)
    repo_report.add(Record(INFORMATION, "Checking repository %s" % repo_path))
    toplevel_folders = sorted(next(os.walk(repo_path))[1])

    for addon_folder in toplevel_folders:
        if addon_folder[0] != '.':
            addon_path = os.path.join(repo_path, addon_folder)
            try:
                addon_report = check_addon.start(addon_path, branch_name, all_repo_addons, pr, config)
                repo_report.add(addon_report)
            except Exception as e:
                repo_report.add(Record(PROBLEM, "Something went wrong. Please see: %s" % e))
    return repo_report



from distutils.version import LooseVersion
from .report import Report
from .record import PROBLEM, Record, WARNING

common_ignore_deps = ['xbmc.metadata.scraper.albums', 'xbmc.metadata.scraper.movies',
                      'xbmc.metadata.scraper.musicvideos', 'xbmc.metadata.scraper.tvshows',
                      'xbmc.metadata.scraper.library', 'xbmc.ui.screensaver', 'xbmc.player.musicviz',
                      'xbmc.python.pluginsource', 'xbmc.python.script', 'xbmc.python.weather', 'xbmc.python.lyrics',
                      'xbmc.python.library', 'xbmc.python.module', 'xbmc.subtitle.module', 'kodi.context.item',
                      'kodi.game.controller', 'xbmc.gui.skin', 'xbmc.webinterface', 'xbmc.addon.repository',
                      'xbmc.pvrclient', 'kodi.gameclient', 'kodi.peripheral', 'kodi.resource', 'xbmc.addon.video',
                      'xbmc.addon.audio', 'xbmc.addon.image', 'xbmc.addon.executable', 'kodi.addon.game',
                      'kodi.audioencoder', 'kodi.audiodecoder', 'xbmc.service', 'kodi.resource.images',
                      'kodi.resource.language', 'kodi.resource.uisounds', 'kodi.resource.games',
                      'kodi.resource.font', 'kodi.inputstream', 'kodi.vfs', 'kodi.imagedecoder', 'xbmc.addon',
                      'xbmc.gui', 'xbmc.json', 'xbmc.metadata', 'xbmc.python', 'script.module.pil']


def check_addon_dependencies(report: Report, repo_addons: dict, parsed_xml, branch_name: str):
    """Check for any new dependencies in addon.xml file and reports them
        :parsed_xml: parsed addon.xml file
        :repo_addons: dictionary having all the addon list of a particular
                        version of kodi
        :branch_name: name of the kodi branch/version
    """

    deps = _get_users_dependencies(parsed_xml)
    ignore = _get_ignore_list(branch_name)

    for required_addon, required_version in deps.items():
        if required_addon not in repo_addons:
            if required_addon not in ignore:
                report.add(Record(
                    PROBLEM, "Required addon %s not available in current repository." % required_addon))
        else:
            available_version = repo_addons[required_addon]

            if required_version is None:
                report.add(Record(WARNING, "Required addon %s does not require a fixed version Available: %s "
                                  % (required_addon, available_version)))
            elif available_version is None:
                report.add(Record(PROBLEM, "Version of %s in required version %s not available"
                                  % (required_addon, required_version)))
            elif LooseVersion(available_version) < LooseVersion(required_version) and (required_addon not in ignore):
                report.add(Record(PROBLEM, "Version mismatch for addon %s. Required: %s, Available: %s "
                                  % (required_addon, required_version, available_version)))


def _get_ignore_list(branch_name):

    if branch_name == "leia":
        common_ignore_deps.extend(["inputstream.adaptive", "inputstream.rtmp",
                                   "script.module.pycryptodome"])
        return common_ignore_deps

    elif branch_name == "krypton":
        common_ignore_deps.extend(["inputstream.adaptive", "inputstream.rtmp"])
        return common_ignore_deps

    else:
        return common_ignore_deps


def _get_users_dependencies(parsed_xml):
    """Gets all the dependencies from a given addon
        :parsed_xml: parsed addon.xml
    """

    return {
        i.get("addon"): i.get("version")
        for i in parsed_xml.findall("requires/import")
    }



import logging
import logging.handlers


class Logger:

    @staticmethod
    def create_logger(debug_filename, logger_name):

        logger = logging.getLogger(logger_name)
        logger.setLevel(logging.DEBUG)
        logger.propagate = False
        formatter = logging.Formatter(fmt='%(asctime)s %(levelname)s:%(name)s:%(funcName)s: %(message)s',
                                      datefmt='%Y-%m-%d %H:%M:%S')

        # DEBUG log to 'kodi-addon-checker.log'
        debug_log_handler = logging.handlers.RotatingFileHandler(debug_filename, encoding='utf-8', mode="w")
        debug_log_handler.setLevel(logging.DEBUG)
        debug_log_handler.setFormatter(formatter)
        logger.addHandler(debug_log_handler)

        return logger



import os
import xml.etree.ElementTree as ET
import requests
import logging
import gzip
from io import BytesIO

from .record import Record, INFORMATION
from .report import Report
from . import check_artwork
from . import check_old_addon
from . import check_dependencies
from . import check_entrypoint
from . import handle_files
from . import check_files
from . import check_string
from . import check_py3_compatibility

REL_PATH = ""
ROOT_URL = "http://mirrors.kodi.tv/addons/{branch}/addons.xml.gz"
LOGGER = logging.getLogger(__name__)


def start(addon_path, branch_name, all_repo_addons, pr, config=None):
    addon_id = os.path.basename(os.path.normpath(addon_path))
    addon_report = Report(addon_id)
    LOGGER.info("Checking add-on %s" % addon_id)
    addon_report.add(Record(INFORMATION, "Checking add-on %s" % addon_id))

    repo_addons = all_repo_addons[branch_name]
    addon_xml_path = os.path.join(addon_path, "addon.xml")
    parsed_xml = ET.parse(addon_xml_path).getroot()

    global REL_PATH
    # Extract common path from addon paths
    # All paths will be printed relative to this path
    REL_PATH = os.path.split(addon_path[:-1])[0]
    addon_xml = check_files.check_addon_xml(addon_report, addon_path, parsed_xml)

    if addon_xml is not None:
        if len(addon_xml.findall("*//broken")) == 0:
            file_index = handle_files.create_file_index(addon_path)

            check_dependencies.check_addon_dependencies(addon_report, repo_addons, parsed_xml, branch_name)

            check_files.check_for_invalid_xml_files(addon_report, file_index)

            check_old_addon.check_for_existing_addon(addon_report, addon_path, all_repo_addons, pr)

            check_files.check_for_invalid_json_files(addon_report, file_index)

            check_artwork.check_artwork(addon_report, addon_path, parsed_xml, file_index)

            max_entrypoint_count = config.configs.get(
                "max_entrypoint_count", 15)
            check_entrypoint.check_complex_addon_entrypoint(
                addon_report, addon_path, parsed_xml, max_entrypoint_count)

            if branch_name not in ['gotham', 'helix', 'isengard', 'jarvis']:
                check_py3_compatibility.Check_Py3_compatibility(addon_report, addon_path)

            if config.is_enabled("check_license_file_exists"):
                # check if license file is existing
                handle_files.addon_file_exists(addon_report, addon_path,
                                               r"^LICENSE\.txt|LICENSE\.md|LICENSE$")

            if config.is_enabled("check_legacy_strings_xml"):
                check_string.check_for_legacy_strings_xml(addon_report, addon_path)

            if config.is_enabled("check_legacy_language_path"):
                check_files.check_for_legacy_language_path(addon_report, addon_path)

            # Kodi 18 Leia + deprecations
            if config.is_enabled("check_kodi_leia_deprecations"):
                check_string.find_blacklisted_strings(addon_report, addon_path,
                                                      ["System.HasModalDialog", "StringCompare", "SubString",
                                                       "IntegerGreaterThan", "ListItem.ChannelNumber",
                                                       "ListItem.SubChannelNumber", "MusicPlayer.ChannelNumber",
                                                       "MusicPlayer.SubChannelNumber", "VideoPlayer.ChannelNumber",
                                                       "VideoPlayer.SubChannelNumber"],
                                                      [], [".py", ".xml"])

            # General blacklist
            check_string.find_blacklisted_strings(addon_report, addon_path, [], [], [])

            check_files.check_file_whitelist(addon_report, file_index, addon_path)
        else:
            addon_report.add(
                Record(INFORMATION, "Addon marked as broken - skipping"))

    return addon_report


def all_repo_addons():
    """Returns a nested dictionary of format:
        {'gotham':{'name_of_addon':'version_of_addon'}}
    """

    branches = ['gotham', 'helix', 'isengard', 'jarvis', 'krypton', 'leia']
    repo_addons = {}

    for branch in branches:
        branch_url = ROOT_URL.format(branch=branch)
        repo_addons[branch] = _get_addons(branch_url)

    return repo_addons


def _get_addons(xml_url: str):
    """Gets addon.xml file for all the version of kodi
        :xml_url: url of the version of kodi
    """
    try:
        gz_file = requests.get(xml_url, timeout=(10, 10)).content
        with gzip.open(BytesIO(gz_file), 'rb') as xml_file:
            content = xml_file.read()
        tree = ET.fromstring(content)

        return {
            a.get("id"): a.get("version")
            for a in tree.findall("addon")
        }
    except requests.exceptions.ReadTimeout as errrt:
        LOGGER.error(errrt)
    except requests.exceptions.ConnectTimeout as errct:
        LOGGER.error(errct)
